//go:build functest
// +build functest

package impl_test

import (
	"fmt"
	"testing"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/geo/impl"
)

func TestGetPoiImpl(t *testing.T) {
	location := impl.MapboxLocation{Longitude: -2.889600, Latitude: 53.188608}
	tags := []string{"", "garden", "cafe", "coffee", "tea", "tea house", "nursery", "bar", ""}
	radius := int32(2000)
	resp := impl.GetPoiImpl(location, tags, radius)
	if resp == nil {
		t.Error("Expected MapboxPoiResponse struct pointer, got nil")
		return
	}

	if resp.Pois == nil {
		t.Error("Expected []MapboxPoi, got nil")
		return
	}

	log.WithContext(ctx).Debug(resp.Pois)
	if len(resp.Pois) != 17 {
		t.Error(fmt.Sprintf("Expected len(resp.Pois) == 17, got %d", len(resp.Pois)))
	}
}

func TestGetIsochronesImpl(t *testing.T) {
	location := impl.MapboxLocation{Longitude: -2.889600, Latitude: 53.188608}
	times := []int32{60, 20, 10, 40, 5}
	resp, err := impl.GetIsochronesImpl(location, times, "driving", 0.05)
	if err != nil {
		t.Error(fmt.Sprintf("Expected error is nil, got %s", err))
		return
	}

	if resp == nil {
		t.Error("Expected MapboxIsochroneResponse struct pointer, got nil")
		return
	}

	if resp.Isochrones == nil {
		t.Error("Expected []MapboxIsochrone, got nil")
		return
	}

	if len(resp.Isochrones) != 5 {
		t.Error(fmt.Sprintf("Expected len(resp.Pois) == 5, got %d", len(resp.Isochrones)))
	}

	if resp.Isochrones[0].Time == 60 {
		if len(resp.Isochrones[0].Contours) != 2 {
			t.Error(fmt.Sprintf("Expected contours count = 2  at 60 minutes, got %d", len(resp.Isochrones[0].Contours)))
		}
	}

	if resp.Isochrones[0].Time == 40 {
		if len(resp.Isochrones[0].Contours) != 2 {
			t.Error(fmt.Sprintf("Expected contours count = 2  at 40 minutes, got %d", len(resp.Isochrones[0].Contours)))
		}
	}
}
