//go:build functest
// +build functest

package impl

import (
	"context"
	"testing"

	"gitlab.mashplace.com/mashplace/backend/geo/impl"

	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"google.golang.org/grpc"
)

const (
	LocalServerAddr         = ":10000"
	DockerComposeServerAddr = "10.251.0.10:10000"
)

func testGeoEndpoints(ctx context.Context, conn *grpc.ClientConn) {
	client := geo.NewGeoServiceClient(conn)

	_, err := client.GetPois(ctx, &geo.PoiRequest{Location: &geo.Location{Longitude: -2.889600, Latitude: 53.188608}, RadiusInMeters: 2000, Tags: []string{"", "garden", "cafe", "coffee", "tea", "tea house", "nursery", "bar", ""}})
	if err != nil {
		log.Panic(err)
	}
	_, err = client.GetIsochrones(ctx, &geo.IsochroneRequest{Location: &geo.Location{Longitude: -2.889600, Latitude: 53.188608}, Times: []int32{20, 10, 40, 60, 5}, Denoise: 1.0, MovingType: geo.IsochroneMovingType_DRIVING})
	if err != nil {
		log.Panic(err)
	}
}

func TestServerInstantiatingDocker(t *testing.T) {
	// t.Skip("disabled")

	s := impl.NewServer()
	go s.Serve(LocalServerAddr)

	conn, err := grpc.Dial(DockerComposeServerAddr, grpc.WithInsecure())
	if err != nil {
		log.Panic(err)
	}

	ctx := context.Background()
	testGeoEndpoints(ctx, conn)
}
