package datagovuk

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type repoLocation struct {
	model.StubRepository
	locations []*model.Location
}

func (r *repoLocation) LocationInsert(ctx context.Context, location *model.Location) error {
	r.locations = append(r.locations, location)

	return nil
}

func TestClinics(t *testing.T) {
	client := &http.Client{
		Transport: &util.MockTripper{
			ResponseBody: []byte("\n17939\xACNAA03\xACClinic\xACUNKNOWN\xACVisible\xACFalse\xAC" +
				"Kirkby Health Suite\xACCherryfield Drive\xAC\xAC\xACLiverpool\xACMerseyside\xAC" +
				"L32 8UR\xAC53.482204437255859\xAC-2.8905689716339111\xAC\xAC\xAC\xAC\n"),
		},
	}

	r := &repoLocation{}

	imp := NewClinics(r, client)

	succ, fail, err := imp.Import(context.Background())
	assert.NoError(t, err)

	assert.EqualValues(t, 1, succ)
	assert.EqualValues(t, 0, fail)

	assert.Len(t, r.locations, 1)

	assert.EqualValues(t, geo.PointType_POINT_CLINIC, r.locations[0].Type)
	assert.EqualValues(t, "Kirkby Health Suite", r.locations[0].Name)
	assert.EqualValues(t, -2.8905689716339111, r.locations[0].Lon)
	assert.EqualValues(t, 53.482204437255859, r.locations[0].Lat)
	assert.EqualValues(t, "Clinic - Kirkby Health Suite\nCherryfield Drive L32 8UR\n", r.locations[0].Description)
}
