package datagovuk

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
)

func TestHospitals(t *testing.T) {
	client := &http.Client{
		Transport: &util.MockTripper{
			ResponseBody: []byte("\n18138\xACNMP01\xACHospital\xACHospital\xACIndependent Sector\xAC" +
				"Visible\xACFalse\xACBridgewater Hospital\xAC120 Princess Road\xAC\xAC\xACManchester\xAC" +
				"Greater Manchester\xACM15 5AT\xAC53.459743499755859\xAC-2.2454688549041748\xACNMP\xAC" +
				"Bridgewater Hospital (Manchester) Ltd\xAC0161 2270000\xAC\xACwww.bridgewaterhospital.com\xAC\n"),
		},
	}

	r := &repoLocation{}

	imp := NewHospitals(r, client)

	succ, fail, err := imp.Import(context.Background())
	assert.NoError(t, err)

	assert.EqualValues(t, 1, succ)
	assert.EqualValues(t, 0, fail)

	assert.Len(t, r.locations, 1)

	assert.EqualValues(t, geo.PointType_POINT_HOSPITAL, r.locations[0].Type)
	assert.EqualValues(t, "Bridgewater Hospital", r.locations[0].Name)
	assert.EqualValues(t, -2.245468854904175, r.locations[0].Lon)
	assert.EqualValues(t, 53.45974349975586, r.locations[0].Lat)
	assert.EqualValues(t, "Hospital Bridgewater Hospital (Manchester) Ltd - Bridgewater Hospital\n"+
		"120 Princess Road M15 5AT\nphone: 0161 2270000\nwebsite: www.bridgewaterhospital.com\n",
		r.locations[0].Description)
}
