package datagovuk

import (
	"context"
	"encoding/base64"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
)

func TestNaptan(t *testing.T) {
	bs, err := base64.StdEncoding.DecodeString(
		`UEsDBAoAAAAAAGhnY1InKZsiVgAAAFYAAAAJABwAU3RvcHMudHh0VVQJAAPzXT9g/10/YHV4CwAB` +
			`BOgDAAAE6AMAADAxMDBCUkExNjkxMQlic3RwYWdhCUJyaXN0b2wgQ2l0eSBDZW50cmUsIFJ1cGVy` +
			`dCBTdHJlZXQgKEhnKQk1MS40NTY3NAktMi41OTQ5CU5VTEwJMwoKUEsBAh4DCgAAAAAAaGdjUicp` +
			`myJWAAAAVgAAAAkAGAAAAAAAAQAAAKSBAAAAAFN0b3BzLnR4dFVUBQAD810/YHV4CwABBOgDAAAE` +
			`6AMAAFBLBQYAAAAAAQABAE8AAACZAAAAAAA=`,
	)

	assert.NoError(t, err)

	client := &http.Client{
		Transport: &util.MockTripper{
			ResponseBody: bs,
		},
	}

	r := &repoLocation{}

	imp := NewNaptan(r, client)

	succ, fail, err := imp.Import(context.Background())
	assert.NoError(t, err)

	assert.EqualValues(t, 1, succ)
	assert.EqualValues(t, 0, fail)

	assert.Len(t, r.locations, 1)

	assert.EqualValues(t, geo.PointType_POINT_BUS_STATION, r.locations[0].Type)
	assert.EqualValues(t, "Bristol City Centre, Rupert Street (Hg)", r.locations[0].Name)
	assert.EqualValues(t, -2.5949, r.locations[0].Lon)
	assert.EqualValues(t, 51.45674, r.locations[0].Lat)
	assert.EqualValues(t, "stop code: bstpaga, vehicle type: 3", r.locations[0].Description)
}
