// Package doogalcouk provides importers from doogal.co.uk
package doogalcouk

import (
	"context"
	"fmt"
	"net/http"
	"strconv"
	"time"

	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"

	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type importerLondonStationsImpl struct {
	repository model.Repository
	client     *http.Client
}

// NewLondonStations returns new importer instance
func NewLondonStations(repo model.Repository, client *http.Client) importer.Importer {
	return &importerLondonStationsImpl{
		repository: repo,
		client:     client,
	}
}

func (imp *importerLondonStationsImpl) ID() string {
	return "mashroom-doogal-co-uk-london-stations"
}

func (imp *importerLondonStationsImpl) Source() string {
	return "https://www.doogal.co.uk/LondonStationsCSV.ashx"
}

func (imp *importerLondonStationsImpl) Import(ctx context.Context) (success, failure int, err error) {
	resp, err := imp.client.Get(imp.Source())
	if err != nil {
		return
	}

	defer func() {
		_ = resp.Body.Close()
	}()

	return util.ImportCSV(resp.Body, ',', func(record []string) (err error) {
		if len(record) < 7 {
			return util.ErrTooShort
		}

		name := record[0]
		latitude := record[3]
		longitude := record[4]
		zone := record[5]
		postcode := record[6]

		lon, err := strconv.ParseFloat(longitude, 64)
		if err != nil {
			return err
		}

		lat, err := strconv.ParseFloat(latitude, 64)
		if err != nil {
			return err
		}

		description := fmt.Sprintf("zone: %s, postcode: %s", zone, postcode)

		m := &model.Location{
			Type:        geo.PointType_POINT_LONDON_STATION,
			Name:        name,
			Description: description,
			Time:        time.Now(),
			Lon:         lon,
			Lat:         lat,
			Source:      imp.Source(),
			Importer:    imp.ID(),
		}

		return imp.repository.LocationInsert(ctx, m)
	})
}
