package doogalcouk

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type repoLocation struct {
	model.StubRepository
	locations []*model.Location
}

func (r *repoLocation) LocationInsert(ctx context.Context, location *model.Location) error {
	r.locations = append(r.locations, location)

	return nil
}

func TestLondonStations(t *testing.T) {
	client := &http.Client{
		Transport: &util.MockTripper{
			ResponseBody: []byte("Addiscombe,534190,166290,51.3798082895941,-0.0732132828267587,\"3,4,5,6\",CR0 7AA\n"),
		},
	}

	r := &repoLocation{}

	imp := NewLondonStations(r, client)

	succ, fail, err := imp.Import(context.Background())
	assert.NoError(t, err)

	assert.EqualValues(t, 1, succ)
	assert.EqualValues(t, 0, fail)

	assert.Len(t, r.locations, 1)

	assert.EqualValues(t, geo.PointType_POINT_LONDON_STATION, r.locations[0].Type)
	assert.EqualValues(t, "Addiscombe", r.locations[0].Name)
	assert.EqualValues(t, -0.0732132828267587, r.locations[0].Lon)
	assert.EqualValues(t, 51.3798082895941, r.locations[0].Lat)
	assert.EqualValues(t, "zone: 3,4,5,6, postcode: CR0 7AA", r.locations[0].Description)
}
