package doogalcouk

import (
	"context"
	"fmt"
	"net/http"
	"strconv"
	"time"

	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type importerUkStationsImpl struct {
	repository model.Repository
	client     *http.Client
}

// NewUkStations returns new importer instance
func NewUkStations(repo model.Repository, client *http.Client) importer.Importer {
	return &importerUkStationsImpl{
		repository: repo,
		client:     client,
	}
}

func (imp *importerUkStationsImpl) ID() string {
	return "mashroom-doogal-co-uk-uk-stations"
}

func (imp *importerUkStationsImpl) Source() string {
	return "https://www.doogal.co.uk/UkStationsCSV.ashx"
}

func (imp *importerUkStationsImpl) Import(ctx context.Context) (success, failure int, err error) {
	resp, err := imp.client.Get(imp.Source())
	if err != nil {
		return
	}

	defer func() {
		_ = resp.Body.Close()
	}()

	return util.ImportCSV(resp.Body, ',', func(record []string) (err error) {
		if len(record) < 8 {
			return util.ErrTooShort
		}

		name := record[0]
		latitude := record[2]
		longitude := record[3]
		owner := record[6]
		srs := record[7]

		lon, err := strconv.ParseFloat(longitude, 64)
		if err != nil {
			return err
		}

		lat, err := strconv.ParseFloat(latitude, 64)
		if err != nil {
			return err
		}

		description := fmt.Sprintf("owner: %s, srs: %s", owner, srs)

		m := &model.Location{
			Type:        geo.PointType_POINT_UK_STATION,
			Name:        name,
			Description: description,
			Time:        time.Now(),
			Lon:         lon,
			Lat:         lat,
			Source:      imp.Source(),
			Importer:    imp.ID(),
		}

		return imp.repository.LocationInsert(ctx, m)
	})
}
