package doogalcouk

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
)

func TestUkStations(t *testing.T) {
	client := &http.Client{
		Transport: &util.MockTripper{
			ResponseBody: []byte("Aber,CF83 1AQ,51.5749691125956,-3.22982905106563,ABE,3813,Arriva Trains Wales," +
				"South Wales Valleys,251108,227270,214996,212546,219868,209622,203456,202486,192788,192180,183136," +
				"169463,136549,134191,0,0,0,0,0,0,0,0,0,0,0,0,0,0\n"),
		},
	}

	r := &repoLocation{}

	imp := NewUkStations(r, client)

	succ, fail, err := imp.Import(context.Background())
	assert.NoError(t, err)

	assert.EqualValues(t, 1, succ)
	assert.EqualValues(t, 0, fail)

	assert.Len(t, r.locations, 1)

	assert.EqualValues(t, geo.PointType_POINT_UK_STATION, r.locations[0].Type)
	assert.EqualValues(t, "Aber", r.locations[0].Name)
	assert.EqualValues(t, -3.22982905106563, r.locations[0].Lon)
	assert.EqualValues(t, 51.5749691125956, r.locations[0].Lat)
	assert.EqualValues(t, "owner: Arriva Trains Wales, srs: South Wales Valleys", r.locations[0].Description)
}
