package tourpedia

import (
	"context"
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type repoLocation struct {
	model.StubRepository
	locations []*model.Location
}

func (r *repoLocation) LocationInsert(ctx context.Context, location *model.Location) error {
	r.locations = append(r.locations, location)

	return nil
}

func TestLondonPOIs(t *testing.T) {
	client := &http.Client{
		Transport: &util.MockTripper{
			ResponseBody: []byte("\"Trafalgar Square\",poi,35556,51.506769192297,-0.12868970632553,London," +
				"\"Admiralty Arch\",4ac518cef964a52012a620e3,0,\"Monument / Landmark\"," +
				"http://tour-pedia.org/api/getPlaceDetails?id=35556," +
				"http://tour-pedia.org/api/getReviewsByPlaceId?placeId=35556\n"),
		},
	}

	r := &repoLocation{}

	imp := NewLondonPOI(r, client)

	succ, fail, err := imp.Import(context.Background())
	assert.NoError(t, err)

	assert.EqualValues(t, 1, succ)
	assert.EqualValues(t, 0, fail)

	assert.Len(t, r.locations, 1)

	assert.EqualValues(t, geo.PointType_POINT_LONDON_POI, r.locations[0].Type)
	assert.EqualValues(t, "Admiralty Arch", r.locations[0].Name)
	assert.EqualValues(t, -0.12868970632553, r.locations[0].Lon)
	assert.EqualValues(t, 51.506769192297, r.locations[0].Lat)
	assert.EqualValues(t, "category: Monument / Landmark", r.locations[0].Description)
}
