//go:generate go-bindata -pkg uktowns -ignore .*\.go .

// Package uktowns provides importer from csv of uk towns
package uktowns

import (
	"bytes"
	"context"
	"strconv"

	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer/util"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type importerImpl struct {
	repository model.Repository
}

// New returns new importer instance
func New(repo model.Repository) importer.Importer {
	return &importerImpl{
		repository: repo,
	}
}

func (imp *importerImpl) ID() string {
	return "mashroom-uk-towns"
}

func (imp *importerImpl) Source() string {
	return "uk-towns.csv"
}

func (imp *importerImpl) Import(ctx context.Context) (success, failure int, err error) {
	bs, err := Asset(imp.Source())
	if err != nil {
		return
	}

	return util.ImportCSV(bytes.NewReader(bs), ',', func(record []string) (err error) {
		if len(record) < 14 {
			return util.ErrTooShort
		}

		lat, err := strconv.ParseFloat(record[7], 64)
		if err != nil {
			return err
		}
		lon, err := strconv.ParseFloat(record[8], 64)
		if err != nil {
			return err
		}

		m := &model.Town{
			Name:     record[1],
			County:   record[2],
			Country:  record[3],
			Postcode: record[10],
			Area:     record[11],
			Region:   record[12],
			Type:     record[13],
			Lat:      lat,
			Lon:      lon,
		}

		return imp.repository.TownInsert(ctx, m)
	})
}
