// Package model provides structures for persistent repository
package model

import "context"

// default parameters
const (
	FindWithinMod     = 5
	FindWithinCluster = 10
)

// Repository interface abstracting specific repository
type Repository interface {
	Migrate(ctx context.Context) error
	ImportSave(ctx context.Context, imp *Import) error
	ImportGetLastByImporterSource(ctx context.Context, importer, source string) (*Import, error)
	LocationInsert(ctx context.Context, location *Location) error
	LocationUpdate(ctx context.Context, location *Location) error
	LocationGetByID(ctx context.Context, id uint64) (*Location, error)
	LocationListWithinRadius(ctx context.Context, radius *RadiusLonLat) ([]*Location, error)
	LocationListWithinBox(ctx context.Context, aabb *AABBLonLat) ([]*Location, error)
	LocationListRelativeWithinRadius(ctx context.Context, radius *RadiusTypeLonLat) ([]*LocationRelative, error)
	GetNearestLocationList(ctx context.Context, lonLatList []LonLatWithLocationTypes) ([][]*LocationRelative, error)
	PostcodeSave(ctx context.Context, postcode *Postcode) error
	PostcodeListByQuery(ctx context.Context, query string, limit int) ([]*Postcode, error)
	PostcodeListWithinRadius(ctx context.Context, radius *RadiusLonLat, limit int) ([]*Postcode, error)
	PostcodeNearbySave(ctx context.Context, nearby *PostcodeNearby) error
	CountryList(ctx context.Context, offset, limit int) ([]*Country, error)
	RegionList(ctx context.Context, offset, limit int) ([]*Region, error)
	RegionListByQuery(ctx context.Context, query string, limit int) ([]*Region, error)
	RegionListByCountry(ctx context.Context, country string) ([]*Region, error)
	RegionListWithinRadius(ctx context.Context, radius *RadiusLonLat, limit int) ([]*Region, error)
	CountyList(ctx context.Context, offset, limit int) ([]*County, error)
	CountyListByQuery(ctx context.Context, query string, limit int) ([]*County, error)
	CountyListByCountryRegion(ctx context.Context, country, region string, limit int) ([]*County, error)
	CountyListWithinRadius(ctx context.Context, radius *RadiusLonLat, limit int) ([]*County, error)
	TownInsert(ctx context.Context, town *Town) error
	TownUpdate(ctx context.Context, town *Town) error
	TownGetByID(ctx context.Context, id uint64) (*Town, error)
	TownList(ctx context.Context, offset, limit int) ([]*Town, error)
	TownListSub(ctx context.Context, id uint64, limit int) ([]*Town, error)
	TownListByQuery(ctx context.Context, query string, limit int) ([]*Town, error)
	TownListByCountry(ctx context.Context, country string) ([]*Town, error)
	TownListByCountryRegionCounty(ctx context.Context, country, region, county string) ([]*Town, error)
	TownListWithinRadius(ctx context.Context, radius *RadiusLonLat, limit int) ([]*Town, error)
	GetNearestTownsForLocationList(ctx context.Context, radiusLonLatList []RadiusLonLat) (towns []*Town, err error)
}
