// Package postgres provides repository implementation for postgresql database
package postgres

import (
	"context"

	"github.com/golang-migrate/migrate/v4"
	"github.com/golang-migrate/migrate/v4/database/postgres"
	bindata "github.com/golang-migrate/migrate/v4/source/go_bindata"
	"github.com/jmoiron/sqlx"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model/postgres/migrations"
)

type repositoryImpl struct {
	DB *sqlx.DB
	repositoryImplImport
	repositoryImplLocation
	repositoryImplPostcode
	repositoryImplCountry
	repositoryImplRegion
	repositoryImplCounty
	repositoryImplTown
}

func prepareNamed(db *sqlx.DB, s string) (r *sqlx.NamedStmt, err error) {
	r, err = db.PrepareNamed(s)

	if err != nil {
		log.WithError(err).Errorf("error preparing %s\n", s)
	}

	return
}

func preparex(db *sqlx.DB, s string) (r *sqlx.Stmt, err error) {
	r, err = db.Preparex(s)

	if err != nil {
		log.Errorf("error %v preparing %s\n", err, s)
	}

	return
}

// NewRepository returns new postgresql repository
func NewRepository(ctx context.Context, db *sqlx.DB) (repos model.Repository, err error) {
	repo := &repositoryImpl{
		DB: db,
	}

	err = repo.Migrate(ctx)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplImport, err = prepareImport(db)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplLocation, err = prepareLocation(db)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplPostcode, err = preparePostcode(db)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplCountry, err = prepareCountry(db)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplRegion, err = prepareRegion(db)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplCounty, err = prepareCounty(db)
	if err != nil {
		return nil, err
	}

	repo.repositoryImplTown, err = prepareTown(db)
	if err != nil {
		return nil, err
	}

	return repo, nil
}

func (repo *repositoryImpl) Migrate(ctx context.Context) error {
	source, err := bindata.WithInstance(bindata.Resource(migrations.AssetNames(), migrations.Asset))
	if err != nil {
		panic(err)
	}

	driver, err := postgres.WithInstance(repo.DB.DB, &postgres.Config{})
	if err != nil {
		panic(err)
	}

	m, err := migrate.NewWithInstance("go-bindata", source, "postgres", driver)
	if err != nil {
		panic(err)
	}

	err = m.Up()

	switch {
	case err == nil:
		log.Println("Migrated.")
	case err == migrate.ErrNoChange:
		log.Println("Migration not required.")
	default:
		return err
	}

	return nil
}
