package postgres

import (
	"context"

	"github.com/jmoiron/sqlx"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

type repositoryImplImport struct {
	save *sqlx.NamedStmt
	get  struct {
		byLastByImporterSource *sqlx.NamedStmt
	}
}

func prepareImport(db *sqlx.DB) (stmt repositoryImplImport, err error) {
	stmt.save, err = prepareNamed(db, queryImportSave)
	if err != nil {
		return
	}

	stmt.get.byLastByImporterSource, err = prepareNamed(db, queryImportGetLastByImporterSource)
	if err != nil {
		return
	}

	return
}

const queryImportSave = `
insert into imports(
  source,
  importer,
  time,
  success,
  failure,
  message
) values (
  :source,
  :importer,
  now(),
  :success,
  :failure,
  :message
) returning
  id,
  source,
  importer,
  time,
  success,
  failure,
  message
`

func (repo *repositoryImplImport) ImportSave(ctx context.Context, imp *model.Import) error {
	err := repo.save.GetContext(ctx, imp, imp)

	return err
}

const queryImportGetLastByImporterSource = `
select
  id,
  source,
  importer,
  time,
  success,
  failure,
  message
from imports where
  importer = :importer and
  source = :source
order by time desc limit 1
`

func (repo *repositoryImplImport) ImportGetLastByImporterSource(
	ctx context.Context,
	importer, source string,
) (m *model.Import, err error) {
	m = &model.Import{
		Importer: importer,
		Source:   source,
	}

	err = repo.get.byLastByImporterSource.GetContext(ctx, m, m)
	if err != nil {
		return nil, err
	}

	return
}
