// Package service provides implementation for geo service
package service

import (
	"fmt"
	"sync"
	"time"

	"gitlab.mashplace.com/mashplace/backend/geo/internal/importer"

	"gitlab.mashplace.com/mashplace/backend/geo/internal/sitemap"

	"gitlab.mashplace.com/mashplace/backend/geo/pkg/mapbox"

	geo "gitlab.mashplace.com/mashplace/backend/geo/api"
	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
)

// Service contract
type Service interface {
	geo.GeoServiceServer
}

// Config for Service
type Config struct {
	Repository       model.Repository
	MapboxClient     mapbox.Client
	SitemapGenerator sitemap.Generator
	SitemapCountries []*sitemap.Country
	Importers        []importer.Importer
	MinImportPeriod  time.Duration
}

func validateConfig(config *Config) error {
	if config == nil {
		return fmt.Errorf("invalid service config")
	}

	if config.Repository == nil {
		return fmt.Errorf("no repository in service config")
	}

	return nil
}

// NewService returns new Service instance
func NewService(config *Config) (Service, error) {
	err := validateConfig(config)
	if err != nil {
		return nil, err
	}

	return &serverImpl{
		Config: *config,
		mutex:  &sync.Mutex{},
	}, nil
}

type serverImpl struct {
	Config
	mutex *sync.Mutex
}
