// Package sitemap provides sitemap generator
package sitemap

import (
	"context"
	"net/http"

	"gitlab.mashplace.com/mashplace/backend/geo/internal/model"
	grabdata "gitlab.mashplace.com/mashplace/backend/grabdata/api"
)

// Country sitemap input
type Country struct {
	Name string
	File string
}

// Config for sitemap generator
type Config struct {
	PublicPagesURL   string
	ArticlesPagesURL string
	SitemapURL       string
	Storage          string
	Repo             model.Repository
	HTTPClient       *http.Client
	GrabdataCli      grabdata.GrabDataServiceClient
}

// Chunker returns data content chunk-by-chunk
type Chunker interface {
	Next() ([]byte, error)
	Close() error
}

// NewGenerator returns new generator
func NewGenerator(config *Config) (Generator, error) {
	return &generatorImpl{
		config: *config,
	}, nil
}

// Generator external interface
type Generator interface {
	Generate(ctx context.Context, countries []*Country) error
	Clear(ctx context.Context) error
	Open(
		ctx context.Context,
		file string,
		chunk []byte,
	) (hash []byte, size int64, mime string, chunker Chunker, err error)
}
